/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.rest;

import lombok.extern.log4j.Log4j2;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 * Responsible for serving WSDL files
 */
@Controller
@RequestMapping("/")
@Log4j2
public class WSDLController {

    private static final String RESOURCE_NAME_DGWS_SERVICE = "/wsdl/ecpr2_dgws_service_inline_2017_10_01.wsdl";
    private static final String RESOURCE_NAME_MASTERDATA = "/wsdl/ecpr2_masterdata_inline_2017_10_01.wsdl";

    @RequestMapping(value = "/wsdl/dgws_service.wsdl", produces = "application/xml")
    @ResponseBody
    public ResponseEntity<String> dgwsService() throws IOException {
        log.debug("wsdl dgws_service request");

        InputStream in = getClass().getResourceAsStream(RESOURCE_NAME_DGWS_SERVICE);
        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
        StringBuilder out = new StringBuilder();
        String line;
        while ((line = reader.readLine()) != null) {
            out.append(line);
        }

        HttpHeaders headers = new HttpHeaders();
        headers.add(HttpHeaders.CACHE_CONTROL, "no-cache");

        return ResponseEntity.ok().headers(headers).body(out.toString());
    }

    @RequestMapping(value = "/wsdl/masterdata.wsdl", produces = "application/xml")
    @ResponseBody
    public ResponseEntity<String> masterdata() throws IOException {
        log.debug("wsdl masterdata request");

        InputStream in = getClass().getResourceAsStream(RESOURCE_NAME_MASTERDATA);
        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
        StringBuilder out = new StringBuilder();
        String line;
        while ((line = reader.readLine()) != null) {
            out.append(line);
        }

        HttpHeaders headers = new HttpHeaders();
        headers.add(HttpHeaders.CACHE_CONTROL, "no-cache");

        return ResponseEntity.ok().headers(headers).body(out.toString());
    }


}
