/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.security;

import dk.sds.ecpr2.dao.StamdataFacade;
import dk.sds.nsp.security.SecurityContext;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

public abstract class AuthorisationNumberAuthenticator implements Authenticator {

    @Autowired private StamdataFacade stamdataFacade;

    @Override
    public boolean authenticate(SecurityContext sc, String requestedRole, OnBehalfOf onBehalfOf) {
        boolean notDoctorNotDentist = !("7170".equals(getEducationCode()) || "5433".equals(getEducationCode()));
        boolean cvrIsNullOrHasNoValue = !sc.getOrganisation().isPresent() || !sc.getOrganisation().get().getIdentifierFormat().equals(SecurityContext.Organisation.OrganisationIdentifierFormat.CVR) || sc.getOrganisation().get().getIdentifier().trim().isEmpty();
        if ((notDoctorNotDentist && cvrIsNullOrHasNoValue)
                || (!sc.getActingUser().isPresent())) {
            return false;
        }

        if (!sc.getActingUser().get().getCredentials().isPresent()
                || !sc.getActingUser().get().getCredentials().get().getEducationCode().isPresent()
                || !sc.getActingUser().get().getCredentials().get().getEducationCode().get().equals(getEducationCode())) {
            return false;
        }
        return true;
    }

    public abstract String getEducationCode();
}
