/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.service.ExportHistory;

import dk.sds.ecpr2.rest.ExportHistoryController;
import dk.sds.ecpr2.IntegrationTest;
import dk.sds.ecpr2.dao.impl.SearchPersonDAOImpl;
import dk.sds.ecpr2.service.historyExport.ExportHistoryServiceImpl;
import dk.sds.ecpr2.util.TimeHelper;
import org.junit.Assert;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.http.HttpStatus;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.util.Date;


public class ExportHistoryControllerTest extends IntegrationTest {

    public static final String EXPORTHISTORY_URL = "/historyExport";

    @Autowired
    private ExportHistoryController serviceUnderTest;

    @Autowired
    ExportHistoryServiceImpl ecpr2ExportService;

    @Test
    public void exportHistory_noErrorInLastExportedHistory_ReturnsOK() throws IOException {

        SearchPersonDAOImpl.firstExecutionCompleted = true;
        ResponseEntity<String> response = serviceUnderTest.historyExport();
        Assert.assertTrue(response.getBody().contains("12/20/2022 19:45:05"));
        Assert.assertEquals(response.getStatusCode(), HttpStatus.OK);
    }

    @Test
    public void exportHistory_ErrorInLastExportedHistoryNewerThanAnHourOld_Returns500() throws Exception {
        Date now = TimeHelper.getCurrentTime();
        // Opdateret test, så der bruges samme data, som nu er tilføjet i DB ved oprettelse
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
        Date previousExportTime = dateFormat.parse("2022-12-20 19:45:05.000");
        ecpr2ExportService.exportHistoryToFiles(previousExportTime.toInstant(), 2);
        ecpr2ExportService.updateExportHistoryTable(now.toInstant(),"some error message");

        SearchPersonDAOImpl.firstExecutionCompleted = true;
        ResponseEntity<String> response = serviceUnderTest.historyExport();
        Assert.assertEquals(response.getStatusCode(), HttpStatus.INTERNAL_SERVER_ERROR);
    }

    @Test
    public void exportHistory_ErrorInAllExportedHistories_Returns500() throws Exception {
        Date now = TimeHelper.getCurrentTime();
        ecpr2ExportService.updateExportHistoryTable(now.toInstant(),"some error message");
        SearchPersonDAOImpl.firstExecutionCompleted = true;
        ResponseEntity<String> response = serviceUnderTest.historyExport();
        Assert.assertEquals(response.getStatusCode(), HttpStatus.INTERNAL_SERVER_ERROR);
    }

    @Test
    public void exportHistory_ErrorInLastExportedHistoryOlderThanAnHourOld_Returns200() throws Exception {
        Date now = TimeHelper.getCurrentTime();
        // Opdateret test, så der bruges samme data, som nu er tilføjet i DB ved oprettelse
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
        Date previousExportTime = dateFormat.parse("2022-12-20 19:45:05.000");

        ecpr2ExportService.exportHistoryToFiles(previousExportTime.toInstant(), 2);
        ecpr2ExportService.updateExportHistoryTable(now.toInstant().minusSeconds(60*65),"some error message");

        SearchPersonDAOImpl.firstExecutionCompleted = true;
        ResponseEntity<String> response = serviceUnderTest.historyExport();
        Assert.assertTrue(response.getBody().contains("12/20/2022 19:45:05"));
        Assert.assertEquals(response.getStatusCode(), HttpStatus.OK);
    }
}